/*
 * Person.cpp
 *
 * LECTURE 22
 * DEMO 4
 */
 
#include "StdAFx.h"

#include <iostream>
#include <string>
using namespace std;

#include "Person.h"

//
// MEMORY MANAGEMENT SECTION
//
// HERE we define a class to store a linked list of Persons.  We define a 
// cast to (Person *) so that Person::operator new may return a cast
// of this structure
//
class PersonElement
{
public:
  operator Person *()
  {
    return &thisData;
  }
  
public:
  Person thisData;
  PersonElement *nextElement;
};

//
// Now our globals to keep track of the memory allocations
//
PersonElement *nextElement = NULL;
PersonElement *lastElement = NULL;
PersonElement *deletedElements = NULL;

//
// Finally, the new and delete overloads
//
void *Person::operator new(size_t size)
{
  // First, make sure size requested matches size we are prepared to deal with
  if (size != sizeof(Person))
  {
    return ::operator new(size);
  }
  
  // First, check the free list
  if (deletedElements)
  {
    void *freePtr = deletedElements;
    deletedElements = deletedElements->nextElement;
    return freePtr;
  }
  
  // Second, check to see if there are any more "array" elements
  if (nextElement < lastElement)
  {
    void *freePtr = nextElement++;
    return freePtr;
  }
  
  // Otherwise, we are out of memory.  Allocate another chunk
  char *newChunk = new char[100000 * sizeof(PersonElement)];
  nextElement = (PersonElement *) newChunk;
  lastElement = nextElement + 100000;
  void *freePtr = nextElement++;
  return freePtr;
}


//
// Now, here's the code to delete an entry (add to our deleted list)
//
void Person::operator delete(void *d)
{
  ((PersonElement *)d)->nextElement = deletedElements;
  deletedElements = (PersonElement *) d;
}
// Here's the definition for the Person::setInfo() member function
void Person::setInfo(string Name,string Addr,string Phone)
{
  name = Name;
  address = Addr;
  phone = Phone;
}

void Person::printInfo()
{
  cout << "Name:  " << name << endl;    // name, address and
  cout << "Addr:  " << address << endl; // phone are defined 
  cout << "Phone: " << phone << endl;   // in the base class
}